/*
 * example1.c
 *
 *  Created on: 18-09-2010
 *      Author: lucck
 */
/* ------------------------------------------------------------------ */
#include <stm32f10x_lib.h>
#include "config.h"
#include "system.h"
/* ------------------------------------------------------------------ */
//! Configure ADC prescaler to 8
#define RCC_CFGR_ADCPRE_8 (3<<14)
//! HSE oscilator control
#define RCC_CR_HSI_ON  (1<<0)
//Number of isix threads
/* ------------------------------------------------------------------ */
//RCC_CFGR2_SET RCC_CFGR2_PREDIV1_DIV1..32
#define RCCCFGR2_M1(x) RCC_CFGR2_PREDIV1_DIV##x
#define RCCCFGR2_M2(x)  RCCCFGR2_M1(x)
#define RCC_CFGR2_SET RCCCFGR2_M2(CONFIG_SYS_DIV)
//RCC_PLLMul_x
#define RCCPLLMUL_M1(x) RCC_PLLMul_##x
#define RCCPLLMUL_M2(x) RCCPLLMUL_M1(x)
#define RCC_PLLMul_SET RCCPLLMUL_M2(CONFIG_PLL_MUL)
/* ------------------------------------------------------------------ */
/** Cortex stm32 System setup
 * Clock and flash configuration for selected rate
 */
static void uc_periph_setup()
{
	//Configure CLK clock
    RCC->CR &= ~RCC_CR_HSEON;
    //Disable Bypass
    RCC->CR &= ~RCC_CR_HSEBYP;
    //Enable high speed oscilator
    RCC->CR  |= RCC_CR_HSEON;
    //Wait for setup HSE
    while(1)
    {
        if(RCC->CR & RCC_CR_HSERDY) break;
    }
    //Configure flash: Prefetch enable and 1 wait state
    FLASH->ACR = FLASH_ACR_PRFTBE | FLASH_ACR_LATENCY_2;

    /* Configure system clocks ALL clocks freq 72MHZ APB1 36MHz
    * PLL x 2
     */
    RCC->CFGR = RCC_CFGR_ADCPRE_8  | RCC_CFGR_PPRE1_DIV2 | RCC_PLLMul_SET |
    		RCC_CFGR_PPRE2_DIV1 | RCC_CFGR_PLLSRC;
    RCC->CFGR2 = RCC_CFGR2_SET;

    // At end disable HSI oscilator for power reduction
    RCC->CR &= ~RCC_CR_HSI_ON;

    //Enable PLL
    RCC->CR |= RCC_CR_PLLON;

    //Wait for PLL sync
    while(1)
    {
	    if(RCC->CR & RCC_CR_PLLRDY)
	    {
	        //PLL as system clock
	        RCC->CFGR |=  RCC_CFGR_SW_PLL;
	        break;
	    }
    }
    //Setup NVIC vector at begin of flash
    SCB->VTOR = NVIC_VectTab_FLASH;
}

/* ------------------------------------------------------------------ */
//! This function is called just before call global constructors
void __external_startup(void)
{

	//Initialize system perhipheral
	uc_periph_setup();

	//1 bit for preemtion priority
	nvic_priority_group(NVIC_PriorityGroup_1);

}

/* ------------------------------------------------------------------ */
